/*
    SDL - Simple DirectMedia Layer
    Copyright (C) 1997-2010 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

    Sam Lantinga
    slouken@libsdl.org
*/
#include "SDL_config.h"

#ifndef _SDL_touch_c_h
#define _SDL_touch_c_h

typedef struct SDL_Touch SDL_Touch;
typedef struct SDL_Finger SDL_Finger;

struct SDL_Finger {
  int id;
  int x;
  int y;
  int z;                      /* for future use */
  int xdelta;
  int ydelta;
  int last_x, last_y;         /* the last reported x and y coordinates */
  int pressure;
};


struct SDL_Touch
{
  
  /* Free the touch when it's time */
  void (*FreeTouch) (SDL_Touch * touch);
  
  /* data common for tablets */
  int pressure_max;
  int pressure_min;
  int tilt;                   /* for future use */
  int rotation;               /* for future use */
  
  /* Data common to all touch */
  int id;
  SDL_Window *focus;
  
  char *name;
  Uint8 buttonstate;
  SDL_bool relative_mode;
  SDL_bool flush_motion;

  int num_fingers;
  SDL_Finger** fingers;
    
  void *driverdata;
};


/* Initialize the touch subsystem */
extern int SDL_TouchInit(void);

/*Get the touch at an index */
extern SDL_Touch *SDL_GetTouchIndex(int index);

/* Get the touch with id = id */
extern SDL_Touch *SDL_GetTouch(int id);

/*Get the finger at an index */
extern SDL_Finger *SDL_GetFingerIndex(SDL_Touch *touch, int index);

/* Get the finger with id = id */
extern SDL_Finger *SDL_GetFinger(SDL_Touch *touch,int id);


/* Add a touch, possibly reattaching at a particular index (or -1),
   returning the index of the touch, or -1 if there was an error. */
extern int SDL_AddTouch(const SDL_Touch * touch, char *name);
                     

/* Remove a touch at an index, clearing the slot for later */
extern void SDL_DelTouch(int index);

/* Set the touch focus window */
extern void SDL_SetTouchFocus(int id, SDL_Window * window);

/* Send a touch motion event for a touch */
extern int SDL_SendTouchMotion(int id, int fingerid,
			       int relative, int x, int y, int z);

/* Send a touch button event for a touch */
extern int SDL_SendTouchButton(int id, Uint8 state, Uint8 button);

/* Shutdown the touch subsystem */
extern void SDL_TouchQuit(void);

/* FIXME: Where do these functions go in this header? */
extern void SDL_ChangeEnd(int id, int end);

#endif /* _SDL_touch_c_h */

/* vi: set ts=4 sw=4 expandtab: */
